# bccpmbld_main.pl - Top script called to compile mainframe files
#
# Authors:  Brandt Onorato, Steve Seifert, Tony Lee, Rich Ralston, Ray Green
# Last updated:  Steve Seifert, 9/14/06
# 	 	 Brandt Onorato 6/06/07
# 		 Jane Huang     6/26/07  
#			 Xiao Lin Zhang 7/06/2009
# 
# (c) Copyright IBM Corp. 2008
#
# The common routines used in all the build scripts are contained in the 
# bccpbmod.pm module.
#
# This script is the top of the call sequence for compiling COBOL and other
# element types like assembler, PLI, and other version of COBOL. The ASM 
# sequence in this main script is a stub to add other element types, just
# copy the COBOL logic for other element types. In addition you must create
# the compile engine and add it into the directory where this main program
# will call, please use COBOL build engine as an example of this, also review
# the flow chart of this build engine script that is documented in the manual
#  
# The file can be executed from the command line.
# After some initialization the script determines the file type
# and calls the appropriate compilation script.  This script supports compiling
# one or more files at a time.
# 
# The "Compile" function in context menu will perform both compile and link-edit
# COBOL program.
# The "Submit Link Job" function in context menu will perform link-edit COBOL module.
#
#
# You must modify this script before attempting to use parameterized build.
# Sections that require modification and the instructions for modifying them
# are flagged with the following comment lines:
#
# ******************************************************************************
# * Customization instructions:                                                *
# * [instructions here]                                                        *
# ****************************************************************************** 

use File::Basename;
use File::Copy;
use Getopt::Std;
use bccpbmod;

my $debug = 1;

# #############################################################################
# Use getopt to process the command arguments
#
# "-i profileUSER_Path" identifies the directory where the profileUSER are located
# "-d dir"   identifies the directory where the elements are located
# "file ..." identifies the file or list of files to be compiled
# "-x"       turns on the expiditor option on the compiler
# "-l"       identifies the link job to be submitted 
#

my $usage = "bccpmbld_main.pl [-x] [-l]  -i profileUSER_Path -d dir file ...";	

my $exped_opt = ""; # Xpediter option
my $sjcl_opt  = ""; # submit link job				
my $compile_1 = "";
my $elem_dir  = ""; # directory where the file resides


if (!getopts('xli:d:') || $opt_h) {         	
    print "$usage \n";
    exit(0);
}

# *******************************************************************************
# * Optional customization:                                                     *
# * Judge OS type to determine the delimiter embedded in path, you can modify it*
# * according to you site's requirement or you can hardcode delimiter here.     *
# ******************************************************************************* 
my $delimiter = '';
my $current_dir = '';

my $OS = $^O;
if ($OS =~/.*Win.*/){	#For Windows operating system
	$delimiter = "\\";
	$current_dir = `cd`;
}elsif ($OS =~/.*n.*x/ or $OS eq "hpux" or $OS eq "solaris"){	# For Unix & Linux operating system
	$delimiter = "/";
	$current_dir = `pwd`;
}elsif ($OS eq "aix"){	#For AIX operating system
	$delimiter = "/";
	$current_dir = `pwd`;
}
chomp $current_dir;


# The argument "-x" test the XPEDITER option that is selected from
# the context menu when compiling an element. This can be performed by using 
# debugging tool like ABENDAID or IBM Debug. When using it the appropriate steps
# within the build script must be modified to reflect this.  Please see
# the COBOL build process script as an example.
# 
$exped_opt = "-x" if ($opt_x);
$sjcl_opt  = "-l" if ($opt_l);   		
$elem_dir  = $opt_d;
$Path_profileUSER = $opt_i;

#diagnose if the path has a tail delimiter
$Size = length($elem_dir);
$position = rindex($elem_dir,$delimiter);
if($position eq ($Size-1)){
	chop $elem_dir;# delete the tail delimiter for further processing
}

print "Main Script: current dir =  $current_dir \n" if $debug;
print "Main Script: Xpediter = $exped_opt \n" if $debug;
print "Main Script: dir_file = $elem_dir \n" if $debug;


# *******************************************************************************
# * Obtain the location of the build script and perlexe set in profileUSER			*
# *******************************************************************************

my $scriptdir = "";   # location of the build scripts
my $bldcobol  = "";   # COBOL build script 
my $linkjob   = "";   # Link Job script

($scriptdir,$lifecycle,$EDfile,$VIEWfile,$listdir,$logdir,$perlexe) = bccpbmod::getprofileUSER($Path_profileUSER,$delimiter);
     
if ($scriptdir) {

        $bldcobol = $scriptdir . $delimiter . "bldcobol.pl";              
        $linkjob = $scriptdir . $delimiter . "linkSubmit.pl";             
        printf( "COBOL Build script:(%s)\n", $bldcobol ) if $debug;  
}
     
    
    # check if $bldcobol = "";
if ($scriptdir eq "" || $bldcobol eq "") {
      	print " Define the build script directory and the Build scripts !\n" ;
   	bccpbmod::pause();
        exit(0);
}


print "Main Build script for Current Directory " .$elem_dir. "\n\n";

# #############################################################################
# check for multiple files to compile and set a variable to be used for
# the compile scripts
# a file list is passed to the build process, this can be one element
# or multiple elements, they do not need to be of the same type. The build 
# engine will determine the proper path through compile process for each element type.
# In this example it supports the selection of multiple elements from a selection list based
# on the users selection from the context of the users view. 
# Also this file list can be used in conjuction with an impact analysis tool
# to compile multiple programs based on what has changed, In this example it assumes
# that this tool already exist, If not one will need to be generated.
#  
my $num_args = @ARGV;
my $compile_1 = "";
$compile_1 = "-1" if ($num_args == 1);


# #############################################################################
# Get the environment variables from the common module
# These variables establish the distributed and mainframe environment
# and must be modified for each installation.  The values for these 
# variables are held in projectED file.
# These varibles are used to point to the Z/OS infrastructure/environment/mapped 
# Life cycle that the project will take. This includes SYSLIB concatenation to be
# used for each environment ie: TEST, QA, PROD. What target loadlibs are to be used
# DB2 subsystem, CICS region, IMS region etc. They are declared at the project level
# the are queried and used here.
# *******************************************************************************
# * Optional customization:                                                     *
# * You can use the optional variables or add new variables based on your site's*
# * needs.                                                                      *
# *******************************************************************************
my ($env, $db2sys, $os390_hostname, $os390_port, $stage,
$copylib1, $copylib2, $copylib3, $copylib4,
$syslib1, $syslib2, $syslib3,
#$syslib4, $steplib1, $steplib2,			 # optional
#$hostpds_prefix, $hostpds_affix, $cbllib,	         # optional 
$loadlib, $c390llib, $dbrmlib, $tgtlib, $db2_owner) = bccpbmod::setEnvironmentVar($EDfile,$lifecycle,$delimiter);
    
# print "Env: $env \n" if $debug;
# print "db2sys: $db2sys \n" if $debug;



# #############################################################################
# Check for connectivity to the mainframe and die if not successful.
bccpbmod::my_die("Server down error.") if (bccpbmod::testServer($os390_hostname, $os390_port) != 0);

# #############################################################################
# Identify each of the file types and separate files from the arguments list
# into arrays by type.  These arrays are passed to the type specific compilation
# scripts.
# These are based on element level attributes that have been set for each element.
# In this example COBOL is used, the ASM is just a stub to be used as an example 
# for adding additional elements types.  Please see the COBOL build process as 
# an example.
my @cobol2_files, @asm_files;
my $no_cobol2_files = 0;
my $no_asm_files = 0;
my $no_unsupported_files = 0;
my $message ="";  # test only -add


# #############################################################################
# This process determines what language types each element is, and that will call
# the apprpriate build process/subroutine/build engine for each element type.
# In this example COBOL is used, the ASM is a STUB to be used to add additional
# build processes.  Please use the COBOL build script/process as an example. The last 
# test determines if the LANG language is supported. The LANG varible is part of the 
# element level attributes that are quieried at the beginning of this process,
#  
# *******************************************************************************
# * Optional customization:                                                     *
# * You can add other language types your site supports and modify the scripts  *
# * based on the needs.                                                         *
# ******************************************************************************* 
foreach $path (@ARGV) {
	my %attrHash = bccpbmod::getAttrs($VIEWfile,$path);

	print "path=$path LANG=$attrHash{'LANG'}\n" if $debug;

	if ($attrHash{'LANG'} eq 'COBOL2') {
		push(@cobol2_files, $path);
	} elsif ($attrHash{'LANG'} eq 'ASM') {
		push(@asm_files, $path);
	} else {
		print "Unsupported language: LANG=$attrHash{'LANG'} path=$path \n";
		$no_unsupported_files++;
	}
}

  

# #############################################################################
# If any of the file type arrays contain filenames, then execute the compilation
# script for each file in the list.
# Call the appropriate build process based on the language
#
$no_cobol2_files = @cobol2_files;
$no_asm_files = @asm_files;

# ------------------------- Compile COBOL files -----------------------------
if ($no_cobol2_files != 0) {
	print "Building COBOL2 files\n";
	bccpbmod::printArray(@cobol2_files);

	foreach $path (@cobol2_files) { 
		my($file, $directory, $suffix) = fileparse($path);   

		print "======================================================= \n";
		print " Building file: " .$file. "\n";
		
		
# Exit if the file is not a COBOL source  		     

		my($testfile,$testdir,$ext) = File::Basename::fileparse($file,qr/\.[^.]*/); 
		         
		if (($ext ne ".cbl") && ($ext ne ".CBL")) {
      		print "ERROR- File extension $ext is not correct !\n" ;
		bccpbmod::pause();
		exit -1;
    }

# Gain attribute type LNKDECK  		
    	my $lnkfound= 0;
    	my %attrHash1 = bccpbmod::getAttrs($VIEWfile,$file);
    	while (($attr, $value) = each %attrHash1) {
      		print "Attribute: $attr\tValue: $value\n";
      		if ($attr eq "LNKDECK"){
        	  $lnkfound = 1;
        	  last;
     		}
    	}

# Exit if 'Submit Link Job' without the type LNKDECK	  

    	if (($sjcl_opt) && ($lnkfound eq 0)){
       		print "ERROR- Attribute type LNKDECK required !\n";
       		bccpbmod::pause();
       		exit -1;
    	} 

  
# ############################################################################
# Cleanup list and log files from the previous execution
		bccpbmod::cleanup($logdir,$delimiter);
		bccpbmod::cleanup($listdir,$delimiter);
    
    if ($sjcl_opt) {
		$logfile = $logdir . $delimiter . $file . "_linkjob.log";
		print $logfile . "\n";
		$message = "Submiting Link Job";
		print $message;
 		print `\"$perlexe\" -I \"$scriptdir\" \"$linkjob\" $compile_1 -i \"$Path_profileUSER\" -d \"$elem_dir\" -f $file > \"$logfile\"`;
    } else {
		$logfile = $logdir . $delimiter . $file . "_compile.log";
		print $logfile . "\n";
		$message = "Compile Build Request";
		print $message;
 		print `\"$perlexe\" -I \"$scriptdir\" \"$bldcobol\" $exped_opt $compile_1 -i \"$Path_profileUSER\" -d \"$elem_dir\" -f $file > \"$logfile\"`;
    } 
    
    
# #############################################################################
# move rccbuild.log to user-defined log dir
#
if(-f $elem_dir . $delimiter . "rccbuild.log"){
	move($elem_dir . $delimiter . "rccbuild.log",$logdir . $delimiter . "rccbuild.log");
}
unlink("." . $delimiter."rccbuild.log");
# #############################################################################
# move new output listing file to user-defined listing dir
#
bccpbmod::movelist($elem_dir,$listdir,$delimiter);

   


		if ($?) {
		 	bccpbmod::pause();
			exit -1;
		}

		print "\n-------------------------------------------------------- \n";
		print "Completed $message of $file\n";
		print "-------------------------------------------------------- \n\n";
	}
}


#---------------------------- Compile ASM files --------------------------
# This is a STUB to add additional File types please use COBOL process
# as an example to add other file types 
# >>>>>>>>>>>>>>>>>>>>>>>>>>>> Not Implemented <<<<<<<<<<<<<<<<<<<<<<<<<<<
if ($no_asm_files != 0) {
	print "Building ASM files:\n";
	bccpbmod::printArray(@asm_files);
}

#------------------------------------------------------
#
# Wait at the end of the compilation for the user to press "any key".  This allows the user to
# review the DOS window output before the window is closed.
bccpbmod::pause();

exit 0;

#######################################################
